"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var types = require("babel-types");
var nejmap_1 = require("nejm/nejmap");
var util_1 = require("./util");
var MODULE_TYPE;
(function (MODULE_TYPE) {
    MODULE_TYPE[MODULE_TYPE["nej"] = 0] = "nej";
    MODULE_TYPE[MODULE_TYPE["text"] = 1] = "text";
    MODULE_TYPE[MODULE_TYPE["custorm"] = 2] = "custorm";
})(MODULE_TYPE || (MODULE_TYPE = {}));
function transformImports(importList, _a) {
    var alias = _a.alias, extName = _a.extName;
    var aliasRe = util_1.genAliasRe(alias);
    var extNameRe;
    if (extName) {
        extNameRe = new RegExp(util_1.escape(extName));
    }
    function _transform(source, specifiers, type) {
        var result = [];
        switch (type) {
            case MODULE_TYPE.nej:
                specifiers.forEach(function (specifier) {
                    result.push({
                        source: nejmap_1.name2source[specifier.imported.name],
                        name: specifier.local.name
                    });
                });
                break;
            case MODULE_TYPE.text:
                specifiers.forEach(function (specifier) {
                    if (source.endsWith('.json')) {
                        source = 'json!' + source;
                    }
                    else {
                        source = 'text!' + source;
                    }
                    result.push({
                        source: source,
                        name: specifier.local.name
                    });
                });
                break;
            case MODULE_TYPE.custorm:
                if (!specifiers.length) {
                    specifiers.push(null);
                }
                specifiers.forEach(function (specifier) {
                    // 处理自定义的后缀名
                    if (extNameRe) {
                        source = source.replace(extNameRe, '');
                    }
                    // 处理 alias
                    var _result = source.match(aliasRe);
                    if (_result) {
                        source = source.replace(_result[0], alias[_result[0]]);
                    }
                    // 相对路径添加 .js 尾缀
                    if (source.startsWith('.') && !source.endsWith('.js')) {
                        source += '.js';
                    }
                    result.push({
                        source: source,
                        name: specifier ? specifier.local.name : null
                    });
                });
                break;
        }
        return result;
    }
    ;
    var result = [];
    importList.forEach(function (impd) {
        var source = impd.source, specifiers = impd.specifiers;
        var sourceName = source.value;
        var defines;
        if (sourceName === 'nejm') {
            defines = _transform(sourceName, specifiers, MODULE_TYPE.nej);
        }
        else if (util_1.TEXT_DEPS.test(sourceName)) {
            defines = _transform(sourceName, specifiers, MODULE_TYPE.text);
        }
        else {
            defines = _transform(sourceName, specifiers, MODULE_TYPE.custorm);
        }
        result = result.concat(defines);
    });
    return result;
}
;
var defaultOptions = {
    extName: '',
    alias: {}
};
function default_1(path, opts) {
    if (opts === void 0) { opts = {}; }
    opts = Object.assign(defaultOptions, opts);
    var node = path.node;
    var importList = [];
    var fnBody = [];
    var exportDeclaration;
    node.body.forEach(function (statement) {
        if (types.isImportDeclaration(statement)) {
            importList.push(statement);
        }
        else if (types.isExportDefaultDeclaration(statement) || types.isExportDefaultSpecifier(statement)) {
            // TODO: 添加所有的 export 类型, 暂时只支持 export default
            exportDeclaration = statement;
        }
        else {
            fnBody.push(statement);
        }
    });
    return {
        nejDefines: transformImports(importList, opts),
        fnBody: fnBody,
        exportDeclaration: exportDeclaration
    };
}
exports.default = default_1;
;
//# sourceMappingURL=transform-es2015-nej.js.map